from __future__ import annotations

import ckan.authz as authz
import ckan.plugins.toolkit as tk
from ckan.lib.helpers import Page
from flask import Blueprint

CONFIG_BASE_TEMPLATE = "ckanext.check_link.report.base_template"
CONFIG_REPORT_URL = "ckanext.check_link.report.url"


DEFAULT_BASE_TEMPLATE = "check_link/base_admin.html"
DEFAULT_REPORT_URL = "/check-link/report/global"


bp = Blueprint("check_link", __name__)


def get_blueprints():
    report_url = tk.config.get(CONFIG_REPORT_URL, DEFAULT_REPORT_URL)
    if report_url:
        bp.add_url_rule(report_url, view_func=report)

    return [bp]


def report():
    if not authz.is_authorized_boolean(
        "check_link_view_report_page", {"user": tk.g.user}, {}
    ):
        return tk.abort(403)

    try:
        page = max(1, tk.asint(tk.request.args.get("page", 1)))
    except ValueError:
        page = 1

    per_page = 10
    reports = tk.get_action("check_link_report_search")(
        {},
        {
            "limit": per_page,
            "offset": per_page * page - per_page,
            "attached_only": True,
            "exclude_state": ["available"],
        },
    )

    base_template = tk.config.get(CONFIG_BASE_TEMPLATE, DEFAULT_BASE_TEMPLATE)
    return tk.render(
        "check_link/report.html",
        {
            "base_template": base_template,
            "page": Page(
                reports["results"],
                page=page,
                item_count=reports["count"],
                items_per_page=per_page,
                presliced_list=True,
            ),
        },
    )



from ckan.logic.schema import validator_args


@validator_args
def url_check(
    not_missing,
    json_list_or_string,
    default,
    convert_to_json_if_string,
    boolean_validator,
):
    return {
        "url": [not_missing, json_list_or_string],
        "patch": [default("{}"), convert_to_json_if_string],
        "save": [default(False), boolean_validator],
        "clear_available": [default(False), boolean_validator],
    }


@validator_args
def resource_check(not_missing, resource_id_exists, boolean_validator, default):
    return {
        "id": [not_missing, resource_id_exists],
        "save": [default(False), boolean_validator],
        "clear_available": [default(False), boolean_validator],
    }


@validator_args
def base_search_check(boolean_validator, default, int_validator):
    return {
        "save": [default(False), boolean_validator],
        "clear_available": [default(False), boolean_validator],
        "include_drafts": [default(False), boolean_validator],
        "include_deleted": [default(False), boolean_validator],
        "include_private": [default(False), boolean_validator],
        "start": [default(0), int_validator],
        "rows": [default(10), int_validator],
    }


@validator_args
def package_check(not_missing, package_id_or_name_exists):
    return dict(base_search_check(), id=[not_missing, package_id_or_name_exists])


@validator_args
def organization_check(not_missing, convert_group_name_or_id_to_id):
    return dict(base_search_check(), id=[not_missing, convert_group_name_or_id_to_id])


@validator_args
def group_check(not_missing, group_id_or_name_exists):
    return dict(base_search_check(), id=[not_missing, group_id_or_name_exists])


@validator_args
def user_check(not_missing, convert_user_name_or_id_to_id):
    return dict(base_search_check(), id=[not_missing, convert_user_name_or_id_to_id])


@validator_args
def search_check(unicode_safe, default):
    return dict(base_search_check(), fq=[default("*:*"), unicode_safe])


@validator_args
def report_save(
    unicode_safe,
    resource_id_exists,
    ignore_missing,
    not_missing,
    default,
    convert_to_json_if_string,
):
    return {
        "id": [ignore_missing, unicode_safe],
        "url": [not_missing, unicode_safe],
        "state": [not_missing, unicode_safe],
        "resource_id": [ignore_missing, resource_id_exists],
        "details": [default("{}"), convert_to_json_if_string],
    }


@validator_args
def report_show(unicode_safe, ignore_missing, resource_id_exists):
    return {
        "id": [ignore_missing, unicode_safe],
        "url": [ignore_missing, unicode_safe],
        "resource_id": [ignore_missing, resource_id_exists],
    }


@validator_args
def report_search(
    ignore_empty, default, int_validator, boolean_validator, json_list_or_string
):
    return {
        "limit": [default(10), int_validator],
        "offset": [default(0), int_validator],
        "exclude_state": [ignore_empty, json_list_or_string],
        "include_state": [ignore_empty, json_list_or_string],
        "attached_only": [default(False), boolean_validator],
        "free_only": [default(False), boolean_validator],
    }


@validator_args
def report_delete(unicode_safe, not_missing):
    return report_show()



"""Create report table

Revision ID: 564f2016af51
Revises:
Create Date: 2022-06-22 06:24:46.742053

"""
import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects.postgresql import JSONB

# revision identifiers, used by Alembic.
revision = "564f2016af51"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "check_link_report",
        sa.Column("id", sa.UnicodeText, primary_key=True),
        sa.Column("url", sa.UnicodeText),
        sa.Column("state", sa.String(20), nullable=False),
        sa.Column(
            "created_at",
            sa.DateTime,
            nullable=False,
            server_default=sa.func.current_timestamp(),
        ),
        sa.Column(
            "resource_id",
            sa.UnicodeText,
            sa.ForeignKey("resource.id"),
            nullable=True,
            unique=True,
        ),
        sa.Column("details", JSONB, nullable=False),
        sa.Index("url_idx", "url"),
        sa.UniqueConstraint("url", "resource_id"),
    )


def downgrade():
    op.drop_table("check_link_report")



from __future__ import annotations

import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit

from . import cli, views
from .logic import action, auth


class CheckLinkPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IAuthFunctions)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.IClick)

    # IConfigurer

    def update_config(self, config_):
        toolkit.add_template_directory(config_, "templates")
        toolkit.add_public_directory(config_, "public")
        toolkit.add_resource("assets", "check_link")

    # IActions
    def get_actions(self):
        return action.get_actions()

    # IAuthFunctions
    def get_auth_functions(self):
        return auth.get_auth_functions()

    # IBlueprint
    def get_blueprint(self):
        return views.get_blueprints()

    # IClick
    def get_commands(self):
        return cli.get_commands()



import ckan.model as model
import pytest

from ckanext.check_link.model import Report


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestReport:
    def test_removed_with_resource(self, resource, faker):
        report = Report(url=faker.url(), resource_id=resource["id"], state="unknown")
        model.Session.add(report)
        model.Session.commit()

        assert report.id

        model.Session.delete(model.Resource.get(resource["id"]))
        model.Session.commit()
        assert not model.Session.query(Report).filter_by(id=report.id).one_or_none()
        assert not Report.by_resource_id(resource["id"])

    def test_by_resource_id(self, report_factory, resource):
        with_resource = report_factory(resource_id=resource["id"])
        report_factory(resource_id=None)

        assert Report.by_resource_id(resource["id"]).id == with_resource["id"]
        assert not Report.by_resource_id(None)

    def test_by_url(self, report_factory):
        first = report_factory()
        second = report_factory(resource_id=None)

        assert not Report.by_url(first["url"])
        assert Report.by_url(second["url"]).id == second["id"]



from __future__ import annotations

import ckan.plugins.toolkit as tk
from ckan.logic import validate

from ckanext.check_link.model import Report
from ckanext.toolbelt.decorators import Collector

from .. import schema

action, get_actions = Collector("check_link").split()


@action
@validate(schema.report_save)
def report_save(context, data_dict):
    tk.check_access("check_link_report_save", context, data_dict)
    sess = context["session"]
    data_dict["details"].update(data_dict.pop("__extras", {}))

    try:
        existing = tk.get_action("check_link_report_show")(context, data_dict)
    except tk.ObjectNotFound:
        report = Report(**{**data_dict, "id": None})
        sess.add(report)
    else:
        report = sess.query(Report).filter(Report.id == existing["id"]).one()
        for k, v in data_dict.items():
            if k == "id":
                continue
            setattr(report, k, v)

    sess.commit()

    return report.dictize(context)


@action
@validate(schema.report_show)
def report_show(context, data_dict):
    tk.check_access("check_link_report_show", context, data_dict)

    if "id" in data_dict:
        report = (
            context["session"]
            .query(Report)
            .filter(Report.id == data_dict["id"])
            .one_or_none()
        )
    elif "resource_id" in data_dict:
        report = Report.by_resource_id(data_dict["resource_id"])
    elif "url" in data_dict:
        report = Report.by_url(data_dict["url"])
    else:
        raise tk.ValidationError(
            {"id": ["One of the following must be provided: id, resource_id, url"]}
        )

    if not report:
        raise tk.ObjectNotFound("Report not found")

    return report.dictize(context)


@action
@validate(schema.report_search)
def report_search(context, data_dict):
    tk.check_access("check_link_report_search", context, data_dict)
    q = context["session"].query(Report)

    if data_dict["free_only"] and data_dict["attached_only"]:
        raise tk.ValidationError(
            {
                "free_only": [
                    "Filters `attached_only` and `free_only` cannot be applied"
                    " simultaneously"
                ]
            }
        )

    if data_dict["free_only"]:
        q = q.filter(Report.resource_id.is_(None))

    if data_dict["attached_only"]:
        q = q.filter(Report.resource_id.isnot(None))

    if "exclude_state" in data_dict:
        q = q.filter(Report.state.notin_(data_dict["exclude_state"]))

    if "include_state" in data_dict:
        q = q.filter(Report.state.in_(data_dict["include_state"]))

    count = q.count()
    q = q.limit(data_dict["limit"]).offset(data_dict["offset"])

    return {
        "count": count,
        "results": [
            r.dictize(dict(context, include_resource=True, include_package=True))
            for r in q
        ],
    }


@action
@validate(schema.report_delete)
def report_delete(context, data_dict):
    tk.check_access("check_link_report_delete", context, data_dict)
    sess = context["session"]

    report = tk.get_action("check_link_report_show")(context, data_dict)
    entity = sess.query(Report).filter(Report.id == report["id"]).one()

    sess.delete(entity)
    sess.commit()
    return entity.dictize(context)
